Attribute VB_Name = "ModSupport"
Option Explicit
'***************************   Start of Constants   ******************************
    Public Const MAX_BUFFER_SIZE = 1024
    Const WRITE_PIPE = 1            'Hex value => 0x01
    Const READ_PIPE = 130           'Hex value => 0x82
    Const ACK = 6                   'Hex value => 0x06
    Const NAK = 21                  'Hex value => 0x15
    Const XON = 17                  'Hex value => 0x11
    Const ASCII_CR = 13             'Hex value => 0x0d
    Const ASCII_SPACE = 32          'Hex value => 0x20
    Const ASCII_OPEN_BRACKET = 60   'Hex value => 0x3c
    Const ASCII_TILDE = 126         'Hex value => 0x7e
'***************************   End of Constatnts   *******************************

'****************   Start of Global Variables   ***************
    Dim ptr_buffer As Long              'Pointer to read/write text buffer
    Dim Str_Size As Long                'Size of text buffer
    Dim Buffer As Variant
    Dim text(0 To (MAX_BUFFER_SIZE - 1)) As Byte               'Read/Write text buffer
    Dim path As String
    Dim Length As Long
    Dim CR, LF, TB As String            'Return,line feed and Tab
'****************   End of Global Variables   ***************

'**************************************************************************************
'*
'* Routine Name  : Initialize()
'*
'* This routine is the equivalent to a startup routine.  When the main form initially
'* loads this routine will execute.
'**************************************************************************************
Public Sub Initialize()

    CR = Chr$(13)
    LF = Chr$(10)
    TB = Chr$(9)
    
End Sub

'**************************************************************************************
'*
'* Routine Name  : Send(text_data As String)
'*
'* This routine writes data out the parallel Port.
'**************************************************************************************
Public Sub Send(text_data As String)
        
    Print #1, text_data; 'parallel port
           
End Sub
'**************************************************************************************
'*
'* Routine Name  : Receive(size As Integer)
'*
'* This routine reads data from the parallel port.
'**************************************************************************************
Public Sub Receive(size As Integer)
    
    Dim count As Integer

    ' At this point, we are expecting to read data from the printer buffer
    ' The amount read is based on the size field, or set to default
        
    'read the print buffer on parallel port
    Buffer = Input(size, #1)
    Length = Len(Buffer)
        
    For count = 1 To Length     'Perhaps call convert
        text(count) = AscB(Mid(Buffer, count, 1))
    Next count
        
    'If nothing is read then Length = 0.
    If Length > 0 Then
        Establish_Status
    Else
        Display_Text ("Nothing read from the print buffer, this time.")
        Display_Text ("If you were expecting data, try again.")
    End If
        
End Sub

'**************************************************************************************
'*
'* Routine Name  : Convert()
'*
'* This Subroutine converts characters to thier Ascii representation
'* for the printer
'**************************************************************************************
Public Sub Convert(itext As String)

    Dim i As Integer
    'Convert string to byte size ascii integers to send to the printer.
    Str_Size = Len(itext)
    For i = 1 To Str_Size
        text(i - 1) = Asc(Mid(itext, i, 1))
    Next i
    
    'Set up Global pointer to data string
    ptr_buffer = VarPtr(text(0))

End Sub

'**************************************************************************************
'*
'* Routine Name  : std_test()
'*
'* Download ticket_count number of copies of any binary file.
'**************************************************************************************
Public Sub std_test(binary_file As String, ticket_count As Integer)
    Dim i As Integer
   
    For i = 1 To ticket_count
        download_file (binary_file)
    Next i

End Sub

'**************************************************************************************
'*
'* Routine Name  : download_file()
'*
'* This routine will download any file through the current open port.
'* The global variable binary_file needs to be initialized before the call.
'**************************************************************************************
Public Sub download_file(binary_file As String)     'DOWNLOAD ANY BINARY FILE
    Dim i As Integer
    Dim byte_count As Integer
    Dim Outgoing_String As String
    Dim Code As String
    
    On Error Resume Next
        
    'initialize
    byte_count = 0
    Outgoing_String = ""
    
    Open binary_file For Binary As #2
    
    Length = LOF(2)
    
    For i = 1 To Length
        
        'Read one byte from input file
        Code = Input(1, #2)
            
        'Buffering the data between calls to Send shortens the
        'download times.
        If byte_count < 128 Then
            Outgoing_String = Outgoing_String + Code
            byte_count = byte_count + 1
        Else
            Send (Outgoing_String)
            Outgoing_String = Code
            byte_count = 1
        End If
            
    Next i
        
    'If there is any data left in the buffer, flush it out.
    If byte_count > 0 Then
        Send (Outgoing_String)
    End If
        
    Close #2

End Sub

'**************************************************************************************
'*
'* Routine Name  : Establish_Status()
'*
'* Once it is assumed the incoming data is status data it will be in
'* the following format
'* Byte #1 (text(0)) contains the length of the string n = 1 to 128
'* Byte #2 thru Byte #n+1 contains status fields which should correspond
'* to the list below.
'*
'*                        0x02 = "Start of Text"
'*                        0x03 = "End of Text"
'*                        0x04 = "End of Transmission"
'*                        0x05 = "Test Button Ticket ACK"
'*                        0x06 = "Ticket ACK"
'*                        0x10 = "Out of Tickets"
'*                        0x11 = "X-On"
'*                        0x12 = "Power On"
'*                        0x13 = "X-Off"
'*                        0x15 = "Ticket NAK"
'*                        0x16 = "Ribbon Low"
'*                        0x17 = "Ribbon Out"
'*                        0x18 = "Ticket Jam"
'*                        0x19 = "Illegal Data"
'*                        0x1A = "Power Up Problem"
'*                        0x1C = "Incomplete Logo Stored in FLASH"
'*
'**************************************************************************************
Public Sub Establish_Status()

    Dim i As Integer                    'counter
    Dim status_response As String       'Status read from Printer
    
    If Length > 0 Then
        Display_Text (CR & LF & "Number" & TB & "Value" & TB & "Meaning")
        Display_Text ("=======" & TB & "=====" & TB & "===============")
    End If
    
    For i = 1 To Length

        'Correspond status number with a text message
        Select Case text(i)
            Case 0 'Hex => 0x00
                status_response = "Null"
            Case 1 'Hex => 0x01
                status_response = "Start of Heading"
            Case 2 'Hex => 0x02
                status_response = "Start of Text"
            Case 3 'Hex => 0x03
                status_response = "End of Text"
            Case 4 'Hex => 0x04
                status_response = "End of Transmission"
            Case 5 'Hex => 0x05
                status_response = "Test Button Ticket ACK"
            Case 6 'Hex => 0x06
                status_response = "Ticket ACK"
            Case 7 'Hex => 0x07
                status_response = "Bell"
            Case 8 'Hex => 0x08
                status_response = "Backspace"
            Case 9 'Hex => 0x09
                status_response = "Horizontal Tab"
            Case 10 'Hex => 0x0A
                status_response = "Line Feed"
            Case 11 'Hex => 0x0B
                status_response = "Vertical Tab"
            Case 12 'Hex => 0x0C
                status_response = "Form Feed"
            Case 13 'Hex => 0x0D
                status_response = "Carriage Return"
            Case 14 'Hex => 0x0E
                status_response = "Shift Out"
            Case 15 'Hex => 0x0F
                status_response = "Shift In"
            Case 16 'Hex => 0x10
                status_response = "Out of Tickets"
            Case 17 'Hex => 0x11
                status_response = "X-On"
            Case 18 'Hex => 0x12
                status_response = "Power On"
            Case 19 'Hex => 0x13
                status_response = "X-Off"
            Case 20 'Hex => 0x14
                status_response = "DC4"
            Case 21 'Hex => 0x15
                status_response = "Ticket NAK"
            Case 22 'Hex => 0x16
                status_response = "Ribbon Low"
            Case 23 'Hex => 0x17
                status_response = "Ribbon Out"
            Case 24 'Hex => 0x18
                status_response = "Ticket Jam"
            Case 25 'Hex => 0x19
                status_response = "Illegal Data"
            Case 26 'Hex => 0x1A
                status_response = "Power Up Problem"
            Case 27 'Hex => 0x1B
                status_response = "Escape"
            Case 28 'Hex => 0x1C
                status_response = "Incomplete Logo Stored in FLASH"
            Case Else 'this data is probably not status data, but a text response
                status_response = ""
        End Select
        'If the byte is a printable ASCII character display the value otherwise
        'display the status response
        If text(i) >= ASCII_SPACE And text(i) <= ASCII_TILDE Then
            Display_Text ("Byte #" & i & TB & text(i) & " " & TB & Chr$(text(i)))
        Else
            Display_Text ("Byte #" & i & TB & text(i) & " " & TB & status_response)
        End If
    Next i
End Sub
' This function will receive a string, check to see if the message text
' box is currently being used and append the string (and CRLF) to whatever is
' currently displayed in the message box
Public Sub Display_Text(str As String)

    'append new string to old text and throw in CR LF
    If FrmBoca.txtmsg.Visible = True Then
        FrmBoca.txtmsg.text = FrmBoca.txtmsg.text & str & CR & LF
    End If
    
End Sub
'Clear the text from the message box
Public Sub Clear_Display()
    FrmBoca.txtmsg.text = ""        'Clear out text
End Sub


