Attribute VB_Name = "ModSupport"
Option Explicit
'***************************   Start of Constants   ******************************
    Const MAX_BUFFER_SIZE = 1024
    Const WRITE_PIPE = 1            'Hex value => 0x01
    Const READ_PIPE = 130           'Hex value => 0x82
    Const ACK = 6                   'Hex value => 0x06
    Const NAK = 21                  'Hex value => 0x15
    Const XON = 17                  'Hex value => 0x11
    Const ASCII_CR = 13             'Hex value => 0x0d
    Const ASCII_SPACE = 32          'Hex value => 0x20
    Const ASCII_OPEN_BRACKET = 60   'Hex value => 0x3c
    Const ASCII_TILDE = 126         'Hex value => 0x7e
'***************************   End of Constatnts   *******************************

'****************   Start of Global Variables   ***************
    Dim ptr_buffer As Long              'Pointer to read/write text buffer
    Dim Str_Size As Long                'Size of text buffer
    Dim Buffer As Variant
    Dim text(0 To (MAX_BUFFER_SIZE - 1)) As Byte               'Read/Write text buffer
    Dim path As String
    Dim Length As Long
    Dim Display_Flag As Boolean
    Public timer_flag As Boolean
    Dim CR, LF, TB As String            'Return,line feed and Tab
    Dim Baud, Bits, Parity As String
    
'****************   End of Global Variables   ***************

'**************************************************************************************
'*
'* Routine Name  : Initialize()
'*
'* This routine is the equivalent to a startup routine.  When the main form initially
'* loads this routine will execute.
'**************************************************************************************
Public Sub Initialize()

    CR = Chr$(13)       'Use this for a carriage return
    LF = Chr$(10)       'Use this for a line feed
    TB = Chr$(9)        'Use this for a horizontal Tab

    'Select com port #1  com1:
    FrmBoca.ComPort.CommPort = 1
    
    'Test printer port to determine its current baud rate, etc.
    Find_Baud
    
End Sub

'**************************************************************************************
'*
'* Routine Name  : Send(text_data As String)
'*
'* This routine writes data out the Serial Port.
'**************************************************************************************
Public Sub Send(text_data As String)
    On Error Resume Next
    
        If (FrmBoca.ComPort.PortOpen <> True) Then
            FrmBoca.ComPort.PortOpen = True
        End If
        FrmBoca.ComPort.Output = text_data
        FrmBoca.ComPort.InputLen = 0
        FrmBoca.ComPort.InBufferCount = 0
        FrmBoca.ComPort.InputMode = comInputModeText
           
End Sub
'**************************************************************************************
'*
'* Routine Name  : Receive(size As Integer)
'*
'* This routine reads data from the Serial port.
'**************************************************************************************
Public Sub Receive(size As Integer)
    
    Dim count, bytes_remaining, i, j As Integer

    On Error Resume Next
        
        ' At this point, we are expecting to read data from the printer buffer
        
        'set 1/10 second status response wait period
        timer_flag = False
        FrmBoca.Timer1.interval = 100
        FrmBoca.Timer1.Enabled = True
        'FrmBoca.ComPort.InBufferCount = 0
        ' Wait 0.1 seconds for data to come back to the serial port.
        Do
            DoEvents
        Loop Until timer_flag = True

        Buffer = FrmBoca.ComPort.Input
        
        Length = Len(Buffer)    'perhaps call convert instead
        For count = 1 To Length
            text(count) = AscB(Mid(Buffer, count, 1))
        Next count
       
        If Display_Flag = True Then
            'If nothing is read then Length = 0.
            If Length > 0 Then
                Establish_Status
            Else
                Display_Text ("Nothing read from the print buffer, this time.")
                Display_Text ("If you were expecting data, try again.")
            End If
        End If
        
End Sub

'**************************************************************************************
'*
'* Routine Name  : Convert()
'*
'* This Subroutine converts characters to thier Ascii representation
'* for the printer
'**************************************************************************************
Public Sub Convert(itext As String)

    Dim i As Integer
    'Convert string to byte size ascii integers to send to the printer.
    Str_Size = Len(itext)
    For i = 1 To Str_Size
        text(i - 1) = Asc(Mid(itext, i, 1))
    Next i
    
    'Set up Global pointer to data string
    ptr_buffer = VarPtr(text(0))

End Sub

'**************************************************************************************
'*
'* Routine Name  : std_test()
'*
'* Download ticket_count number of copies of any binary file.
'**************************************************************************************
Public Sub std_test(binary_file As String, ticket_count As Integer)
    Dim i As Integer
   
    For i = 1 To ticket_count
        download_file (binary_file)
    Next i

End Sub

'**************************************************************************************
'*
'* Routine Name  : download_file()
'*
'* This routine will download any file through the current open port.
'* The global variable binary_file needs to be initialized before the call.
'**************************************************************************************
Public Sub download_file(binary_file As String)     'DOWNLOAD ANY BINARY FILE
    Dim i As Integer
    Dim byte_count As Integer
    Dim Outgoing_String As String
    Dim Code As String
    
    On Error Resume Next
        
    'initialize
    byte_count = 0
    Outgoing_String = ""
    
    Open binary_file For Binary As #2
    
    Length = LOF(2)
    
    For i = 1 To Length
        
        'Read one byte from input file
        Code = Input(1, #2)
            
        'Buffering the data between calls to Send shortens the
        'download times.
        If byte_count < 128 Then
            Outgoing_String = Outgoing_String + Code
            byte_count = byte_count + 1
        Else
            Send (Outgoing_String)
            Outgoing_String = Code
            byte_count = 1
        End If
            
    Next i
        
    'If there is any data left in the buffer, flush it out.
    If byte_count > 0 Then
        Send (Outgoing_String)
    End If
        
    Close #2

End Sub

'**************************************************************************************
'*
'* Routine Name  : Establish_Status()
'*
'* Once it is assumed the incoming data is status data it will be in
'* the following format
'* Byte #1 (text(0)) contains the length of the string n = 1 to 128
'* Byte #2 thru Byte #n+1 contains status fields which should correspond
'* to the list below.
'*
'*                        0x02 = "Start of Text"
'*                        0x03 = "End of Text"
'*                        0x04 = "End of Transmission"
'*                        0x05 = "Test Button Ticket ACK"
'*                        0x06 = "Ticket ACK"
'*                        0x10 = "Out of Tickets"
'*                        0x11 = "X-On"
'*                        0x12 = "Power On"
'*                        0x13 = "X-Off"
'*                        0x15 = "Ticket NAK"
'*                        0x16 = "Ribbon Low"
'*                        0x17 = "Ribbon Out"
'*                        0x18 = "Ticket Jam"
'*                        0x19 = "Illegal Data"
'*                        0x1A = "Power Up Problem"
'*                        0x1C = "Incomplete Logo Stored in FLASH"
'*
'**************************************************************************************
Public Sub Establish_Status()

    Dim i As Integer                    'counter
    Dim status_response As String       'Status read from Printer
    
    'Display details
    If Length > 0 Then
        Display_Text (CR & LF & "Number" & TB & "Value" & TB & "Meaning")
        Display_Text ("=======" & TB & "=====" & TB & "===============")
    End If
    
    'Perform loop for each bytes in the string
    For i = 1 To Length

        'Correspond status number with a text message
        Select Case text(i)
            Case 0 'Hex => 0x00
                status_response = "Null"
            Case 1 'Hex => 0x01
                status_response = "Start of Heading"
            Case 2 'Hex => 0x02
                status_response = "Start of Text"
            Case 3 'Hex => 0x03
                status_response = "End of Text"
            Case 4 'Hex => 0x04
                status_response = "End of Transmission"
            Case 5 'Hex => 0x05
                status_response = "Test Button Ticket ACK"
            Case 6 'Hex => 0x06
                status_response = "Ticket ACK"
            Case 7 'Hex => 0x07
                status_response = "Bell"
            Case 8 'Hex => 0x08
                status_response = "Backspace"
            Case 9 'Hex => 0x09
                status_response = "Horizontal Tab"
            Case 10 'Hex => 0x0A
                status_response = "Line Feed"
            Case 11 'Hex => 0x0B
                status_response = "Vertical Tab"
            Case 12 'Hex => 0x0C
                status_response = "Form Feed"
            Case 13 'Hex => 0x0D
                status_response = "Carriage Return"
            Case 14 'Hex => 0x0E
                status_response = "Shift Out"
            Case 15 'Hex => 0x0F
                status_response = "Shift In"
            Case 16 'Hex => 0x10
                status_response = "Out of Tickets"
            Case 17 'Hex => 0x11
                status_response = "X-On"
            Case 18 'Hex => 0x12
                status_response = "Power On"
            Case 19 'Hex => 0x13
                status_response = "X-Off"
            Case 20 'Hex => 0x14
                status_response = "DC4"
            Case 21 'Hex => 0x15
                status_response = "Ticket NAK"
            Case 22 'Hex => 0x16
                status_response = "Ribbon Low"
            Case 23 'Hex => 0x17
                status_response = "Ribbon Out"
            Case 24 'Hex => 0x18
                status_response = "Ticket Jam"
            Case 25 'Hex => 0x19
                status_response = "Illegal Data"
            Case 26 'Hex => 0x1A
                status_response = "Power Up Problem"
            Case 27 'Hex => 0x1B
                status_response = "Escape"
            Case 28 'Hex => 0x1C
                status_response = "Incomplete Logo Stored in FLASH"
            Case Else 'this data is probably not status data, but a text response
                status_response = ""
        End Select
        
        'If the byte is a printable ASCII character display the value otherwise
        'display the status response
        If text(i) >= ASCII_SPACE And text(i) <= ASCII_TILDE Then
            Display_Text ("Byte #" & i & TB & text(i) & " " & TB & Chr$(text(i)))
        Else
            Display_Text ("Byte #" & i & TB & text(i) & " " & TB & status_response)
        End If
    Next i
End Sub
' This function will receive a string, check to see if the message text
' box is currently being used and append the string (and CRLF) to whatever is
' currently displayed in the message box
Public Sub Display_Text(str As String)

    'append new string to old text and throw in CR LF
    If FrmBoca.txtmsg.Visible = True Then
        FrmBoca.txtmsg.text = FrmBoca.txtmsg.text & str & Chr$(13) & Chr$(10)
    End If
    
End Sub
'Clear the text from the message box
Public Sub Clear_Display()
    FrmBoca.txtmsg.text = ""        'Clear out text
End Sub
Public Sub Find_Baud()
Dim i As Integer
Dim j As String
Dim Command As String

Dim MAX_CHOICES As Integer

MAX_CHOICES = 28

'printer_busy
For i = 1 To MAX_CHOICES
    Select Case i
        Case 1
            Baud = "9600"
            Parity = "E"
            Bits = "7"
        Case 2
            Baud = "9600"
            Parity = "O"
            Bits = "7"
        Case 3
            Baud = "9600"
            Parity = "N"
            Bits = "8"
        Case 4
            Baud = "115200"
            Parity = "E"
            Bits = "7"
        Case 5
            Baud = "115200"
            Parity = "O"
            Bits = "7"
            'ComPort.Settings = "115200,O,7,1"   'not on factory menu
        Case 6
            Baud = "115200"
            Parity = "N"
            Bits = "8"
        Case 7
            Baud = "1200"
            Parity = "E"
            Bits = "7"
        Case 8
            Baud = "1200"
            Parity = "O"
            Bits = "7"
        Case 9
            Baud = "1200"
            Parity = "N"
            Bits = "8"
        Case 10
            Baud = "2400"
            Parity = "E"
            Bits = "7"
        Case 11
            Baud = "2400"
            Parity = "O"
            Bits = "7"
        Case 12
            Baud = "2400"
            Parity = "N"
            Bits = "8"
        Case 13
            Baud = "4800"
            Parity = "E"
            Bits = "7"
        Case 14
            Baud = "4800"
            Parity = "O"
            Bits = "7"
        Case 15
            Baud = "4800"
            Parity = "N"
            Bits = "8"
        Case 16
            Baud = "19200"
            Parity = "E"
            Bits = "7"
        Case 17
            Baud = "19200"
            Parity = "O"
            Bits = "7"
        Case 18
            Baud = "19200"
            Parity = "N"
            Bits = "8"
        Case 19
            Baud = "28800"
            Parity = "E"
            Bits = "7"
        Case 20
            Baud = "28800"
            Parity = "O"
            Bits = "7"
        Case 21
            Baud = "28800"
            Parity = "N"
            Bits = "8"
        Case 22
            Baud = "38400"
            Parity = "E"
            Bits = "7"
        Case 23
            Baud = "38400"
            Parity = "O"
            Bits = "7"
        Case 24
            Baud = "38400"
            Parity = "N"
            Bits = "8"
        Case 25
            Baud = "57600"
            Parity = "E"
            Bits = "7"
        Case 26
            Baud = "57600"
            Parity = "O"
            Bits = "7"
        Case 27
            Baud = "57600"
            Parity = "N"
            Bits = "8"
        Case Else 'If we can't figure it out set default as 9600,N,8,1
            Baud = "9600"
            Parity = "N"
            Bits = "8"
            FrmBoca.TxtBaud.text = Baud
            FrmBoca.TxtBits.text = Bits
            FrmBoca.TxtParity.text = Parity
    End Select
    
    'build string that contains serial port settings
    Command = Baud + "," + Parity + "," + Bits + ",1"

    FrmBoca.ComPort.InputLen = 0
    FrmBoca.ComPort.InputMode = comInputModeText
    FrmBoca.ComPort.Settings = Command              'Establish Com Port Rate
    Send ("<S1>")                                   'Send the test command
    
    'When reading buffer text(0) contains the length of the rest of the string
    Display_Flag = False
    Receive (2)                                     'Looking for X-On => ASCII 17
    Display_Flag = True
    
    'If nothing is read then Length will still = 0.
    'If something was read, is it X-On?
    If Length > 0 And text(1) = XON Then
        FrmBoca.TxtBaud.text = Baud
        FrmBoca.TxtBits.text = Bits
        FrmBoca.TxtParity.text = Parity
        i = MAX_CHOICES                             'got a response so lets exit this loop
    End If

Next i
'printer_ready
End Sub
'**************************************************************************************
'*
'* Routine Name  : handshake()
'*
'* This routine handles handshaking for the serial port
'**************************************************************************************
Public Sub handshake()
    
    FrmBoca.ComPort.Handshaking = 0     'TURN OFF ready line/x-off detection
    FrmBoca.ComPort.OutBufferSize = 512 'needed by windows 95
    If (FrmBoca.ComPort.PortOpen = False) Then FrmBoca.ComPort.PortOpen = True
    Send ("<" + Chr$(0) + "S" + Chr$(0) + "2" + Chr$(0) + ">" + Chr$(0))
    Send ("<S2>")
    FrmBoca.ComPort.InputLen = 0
    FrmBoca.ComPort.InputMode = comInputModeText
        
    FrmBoca.ComPort.PortOpen = False
End Sub
'**************************************************************************************
'*
'* Routine Name  : Busy()
'*
'* This routine updates screen and calls pause.
'**************************************************************************************
Public Sub Busy(duration As Double)

    'Limit the pause time to 10 seconds max
    If duration < 0 Or duration > 10 Then
        duration = 2            '2 second delay
    End If
    
    'Update Screen
    'printer_busy
    Pause (duration)           'pause for "duration" # of seconds
    'printer_ready
End Sub
'Initiate a pause on the screen.  The real passed represents
'the number of seconds to pause.  As a failsafe a message box
'will be used if count is less than or equal to zero
Public Sub Pause(seconds As Double)
    Dim interval As Integer
    
    On Error Resume Next

    interval = seconds * 1000
    
    If interval <= 0 Then
        MsgBox "You called for a Pause, Sir?", vbOKOnly, "Enter to Continue"
    Else
        timer_flag = False
        FrmBoca.Timer1.interval = interval
        FrmBoca.Timer1.Enabled = True
        ' Wait for timeout.
        Do
            DoEvents
        Loop Until timer_flag = True
        FrmBoca.Timer1.Enabled = False
    End If
End Sub



