Attribute VB_Name = "ModSupport"
Option Explicit
'***************************   Start of Constants   ******************************
    Const MAX_BUFFER_SIZE = 1024
    Const USB_MAX_PACKET_SIZE = 32
    Const USB_MAX_PIPE_NUMBER = 16
    Const WRITE_PIPE As Integer = 1            'Hex value => 0x02 0x01
    Const READ_PIPE As Integer = 130           'Hex value => 0x81 0x82
    Const ACK = 6                   'Hex value => 0x06
    Const NAK = 21                  'Hex value => 0x15
    Const XON = 17                  'Hex value => 0x11
    Const ASCII_CR = 13             'Hex value => 0x0d
    Const ASCII_SPACE = 32          'Hex value => 0x20
    Const ASCII_OPEN_BRACKET = 60   'Hex value => 0x3c
    Const ASCII_TILDE = 126         'Hex value => 0x7e
'***************************   End of Constatnts   *******************************

'** DO NOT CHANGE THIS STRUCTURE.  IT MATCHES THE STRUCTURE IN THE DLL  **
Public Type USB_PRINTER
    hUSB As Long                'pointer to printer config
    Serial_Number As Long       'Printer serial number
    Product_ID As Long          'Product ID number
End Type
Public Printer_Table(1 To 127) As USB_PRINTER


'***************************   Start of Global Variables   ***********************
    Dim ptr_buffer As Long                          'Pointer to read/write text buffer
    Public ptr_USB As Long                          'Pointer to the USB Hardware Configuration Data
    Dim Str_Size As Long                            'Size of text buffer
    Dim Buffer As Variant
    Dim text(0 To (MAX_BUFFER_SIZE - 1)) As Byte    'Read/Write text buffer
    Dim path As String
    Dim length As Double
    Dim usbptr_string As String
    Dim CR, LF, TB As String                        'Return,line feed and Tab
    Dim MsgVar
    Dim bytes_read As Long
    Dim bytes_written As Long
    Dim message As String
    Public pTable As Long
    Public USB_printer_count As Integer
    Public current_printer As Integer
    Public Form_Number As Integer
'***************************   End of Global Variables   ***********************

'***************************   Start of Boca_VB1.dll USB API's/Subroutines   ********
    Public Declare Sub Initialize_USB Lib "Boca_VB.dll" _
     Alias "RegisterWinDriver" ()
    
    Public Declare Function Open_USB Lib "Boca_VB.dll" _
     Alias "USB_LocateAndOpenDevice" _
     (ByVal fUseDefault As Boolean, ByVal index As Integer) As Long
     
    Public Declare Sub Close_USB Lib "Boca_VB.dll" _
     Alias "USB_Close" (ByVal hUSB As Long)
    
    Public Declare Function Read_USB Lib "Boca_VB.dll" _
      (ByVal hUSB As Long, ByVal pBuffer As Long, _
       ByVal dwSize As Long, ByVal Pipe As Integer) As Long
    
    Public Declare Function Write_USB Lib "Boca_VB.dll" _
      (ByVal hUSB As Long, ByVal pBuffer As Long, _
       ByVal dwSize As Long, ByVal Pipe As Integer) As Long
    
    Public Declare Sub Timeout_USB Lib "Boca_VB.dll" _
     Alias "USB_Timeout" (ByVal pBuffer As Long, ByVal Timeout As Integer)
    
    Public Declare Function USB_Status Lib "Boca_VB.dll" _
     Alias "Status_USB" (ByVal hUSB As Long) As Long
     
    Public Declare Function Find_Printers Lib "Boca_VB.dll" _
    Alias "USB_Find" (ByVal hUSB_Table As Long) As Integer
'*********************   End of USB Subroutines   ********************

'**************************************************************************************
'*
'* Routine Name  : Initialize()
'*
'* This routine is the equivalent to a startup routine.  When the main form initially
'* loads this routine will execute.
'**************************************************************************************
Public Sub Initialize()
Dim temp(0 To 1) As Byte
Dim ptr As Long              'Pointer to read/write text buffer
Dim Status As Long
Dim auto As Boolean
Dim i, index As Integer
Dim ptr1, ptr2 As String
Dim Name As String * 64        'Product Name

    CR = Chr$(13)       'Use this for a carriage return
    LF = Chr$(10)       'Use this for a line feed
    TB = Chr$(9)        'Use this for a horizontal Tab
    
    'Establish default directory to be the application directory
    FrmBoca.CommonDialog1.InitDir = App.path
    ChDir App.path

    'This routine enables the USB Communication thru the dll.  Without
    'this call, no communication.
    Call Initialize_USB

    'Establish location of a Boca Systems USB printers, should at least one be connected.
    'if auto=True then index is irrelavent, a printer will be automatically selected
    'if auto=False, then the printer selected is based on the index.
    'if three printers found the indexes will run from 0 to 2.
    auto = True                             'select first printer found
    index = 0
    current_printer = 1
    Call Scan_USB                           'Scan for all Boca USB Printers
    Call Pick_USB(ByVal auto, ByVal index)  'Automatically pick the first one found
End Sub

'**************************************************************************************
'*
'* Routine Name  : Shutdown()
'*
'* This routine closes the USB Port.
'**************************************************************************************
Public Sub Shutdown()
    
    'Break link with Boca Systems USB printer, if one exists.
    If ptr_USB <> 0 Then
        Call Close_USB(ByVal ptr_USB)
        ptr_USB = 0
        Printer_Table(current_printer).hUSB = ptr_USB
    End If
      
End Sub

'**************************************************************************************
'*
'* Routine Name  : Send(text_data As String)
'*
'* This routine writes data out the USB Port.
'**************************************************************************************
Public Sub Send(text_data As String)
        
    ' Check the USB pointer for readiness
    If ptr_USB = 0 Then
            
        Display_Text ("Unable to Locate Boca Systems USB Printer")
        
    Else
            
        'Display_Text ("A Boca Systems USB Printer has been located")
            
        'This string needs to be converted into a byte array of ascii
        'integers to send to the printer. Convert will also initilize
        'ptr_buffer and Str_Size.
        Call Convert(text_data)
            
        'Write converted text string and read for status response
        'Need to convert these to functions to receive return values as Longs???
        bytes_written = Write_USB(ByVal ptr_USB, ByVal ptr_buffer, _
                          ByVal Str_Size, ByVal WRITE_PIPE)
        
    End If

End Sub
'**************************************************************************************
'*
'* Routine Name  : Receive(size As Integer)
'*
'* This routine reads data from the USB port, ie. the print buffer.
'**************************************************************************************
Public Sub Receive(size As Integer)
    
    ' Check the USB pointer for readiness
    If ptr_USB = 0 Then
            
        Display_Text ("Unable to Locate Boca Systems USB Printer")
        
    Else
            
        'Display_Text ("A Boca Systems USB Printer has been located")
            
        ' At this point, we are expecting to read data from the printer buffer
        ' The amount read is based on the size field, or set to default
        If (size > 0 And size < USB_MAX_PACKET_SIZE) Then
            Str_Size = size
        Else
            Str_Size = USB_MAX_PACKET_SIZE
        End If
                
        'When reading the USB port,  text(0) will contain the return byte
        'count for the rest of the string.
        text(0) = 0
        ptr_buffer = VarPtr(text(0))
        bytes_read = Read_USB(ByVal ptr_USB, ByVal ptr_buffer, _
                         ByVal Str_Size, ByVal READ_PIPE)
            
        'If nothing is read then text(0) will still = 0.
        length = text(0)
        If length > 0 Then
            Establish_Status
        Else
            Display_Text ("Nothing read from the print buffer, this time.  If you were expecting data, try again.")
        End If
    End If
           
End Sub

'**************************************************************************************
'*
'* Routine Name  : Convert()
'*
'* This Subroutine converts characters to thier Ascii representation
'* for the printer
'**************************************************************************************
Public Sub Convert(itext As String)

    Dim i As Integer
    
    'Convert string to byte size ascii integers to send to the printer.
    Str_Size = Len(itext)
    For i = 1 To Str_Size
        text(i - 1) = Asc(Mid(itext, i, 1))
    Next i
    
    'Set up Global pointer to data string
    ptr_buffer = VarPtr(text(0))

End Sub

'**************************************************************************************
'*
'* Routine Name  : std_test()
'*
'* Download ticket_count number of copies of any binary file.
'**************************************************************************************
Public Sub std_test(binary_file As String, ticket_count As Integer)
    
    Dim i As Integer
   
    For i = 1 To ticket_count
        download_file (binary_file)
    Next i

End Sub

'**************************************************************************************
'*
'* Routine Name  : download_file()
'*
'* This routine will download any file through the current open port.
'* The global variable binary_file needs to be initialized before the call.
'**************************************************************************************
Public Sub download_file(binary_file As String)     'DOWNLOAD ANY BINARY FILE
    
    Dim i As Long
    Dim byte_count As Integer
    Dim Outgoing_String As String
    Dim Code As String
    
    'On Error Resume Next
    On Error GoTo Err_Handler
        
    'initialize
    byte_count = 0
    Outgoing_String = ""
    
    'Open App.path & "\" & binary_file For Binary As #2
    Open binary_file For Binary As #2
    
    length = LOF(2)
    
    For i = 1 To length
        
        'Read one byte from input file
        Code = Input(1, #2)
            
        'Buffering the data between calls to Send shortens the
        'download times.
        If byte_count < 128 Then
            Outgoing_String = Outgoing_String + Code
            byte_count = byte_count + 1
        Else
            Send (Outgoing_String)
            Outgoing_String = Code
            byte_count = 1
        End If
            
    Next i
        
    'If there is any data left in the buffer, flush it out.
    If byte_count > 0 Then
        Send (Outgoing_String)
    End If
        
    Close #2
    Exit Sub

Err_Handler:
    MsgVar = MsgBox("Download File", 64, Err.Description)
           

End Sub

'**************************************************************************************
'*
'* Routine Name  : Establish_Status()
'*
'* Once it is assumed the incoming data is status data it will be in
'* the following format
'* Byte #1 (text(0)) contains the length of the string n = 1 to 128
'* Byte #2 thru Byte #n+1 contains status fields which should correspond
'* to the list below.
'*
'*                        0x02 = "Start of Text"
'*                        0x03 = "End of Text"
'*                        0x04 = "End of Transmission"
'*                        0x05 = "Test Button Ticket ACK"
'*                        0x06 = "Ticket ACK"
'*                        0x10 = "Out of Tickets"
'*                        0x11 = "X-On"
'*                        0x12 = "Power On"
'*                        0x13 = "X-Off"
'*                        0x15 = "Ticket NAK"
'*                        0x16 = "Ribbon Low"
'*                        0x17 = "Ribbon Out"
'*                        0x18 = "Ticket Jam"
'*                        0x19 = "Illegal Data"
'*                        0x1A = "Power Up Problem"
'*                        0x1C = "Incomplete Logo Stored in FLASH"
'*
'**************************************************************************************
Public Sub Establish_Status()

    Dim i As Integer                    'counter
    Dim status_response As String       'Status read from Printer
    Dim counter As Integer
    
    counter = 1
    Display_Text (length & " byte(s) read from printer buffer.")
    'Perform loop for each byte in the string
    For i = 1 To length

        'Correspond status number with a text message
        Select Case text(i)
            Case 0 'Hex => 0x00
                status_response = "Null"
            Case 1 'Hex => 0x01
                status_response = "Start of Heading"
            Case 2 'Hex => 0x02
                status_response = "Start of Text"
            Case 3 'Hex => 0x03
                status_response = "End of Text"
            Case 4 'Hex => 0x04
                status_response = "End of Transmission"
            Case 5 'Hex => 0x05
                status_response = "Test Button Ticket ACK"
            Case 6 'Hex => 0x06
                status_response = "Ticket ACK"
            Case 7 'Hex => 0x07
                status_response = "Bell"
            Case 8 'Hex => 0x08
                status_response = "Backspace"
            Case 9 'Hex => 0x09
                status_response = "Horizontal Tab"
            Case 10 'Hex => 0x0A
                status_response = "Line Feed"
            Case 11 'Hex => 0x0B
                status_response = "Vertical Tab"
            Case 12 'Hex => 0x0C
                status_response = "Form Feed"
            Case 13 'Hex => 0x0D
                status_response = "Carriage Return"
            Case 14 'Hex => 0x0E
                status_response = "Shift Out"
            Case 15 'Hex => 0x0F
                status_response = "Shift In"
            Case 16 'Hex => 0x10
                status_response = "Out of Tickets"
            Case 17 'Hex => 0x11
                status_response = "X-On"
            Case 18 'Hex => 0x12
                status_response = "Power On"
            Case 19 'Hex => 0x13
                status_response = "X-Off"
            Case 20 'Hex => 0x14
                status_response = "DC4"
            Case 21 'Hex => 0x15
                status_response = "Ticket NAK"
            Case 22 'Hex => 0x16
                status_response = "Ribbon Low"
            Case 23 'Hex => 0x17
                status_response = "Ribbon Out"
            Case 24 'Hex => 0x18
                status_response = "Ticket Jam"
            Case 25 'Hex => 0x19
                status_response = "Illegal Data"
            Case 26 'Hex => 0x1A
                status_response = "Power Up Problem"
            Case 27 'Hex => 0x1B
                status_response = "Escape"
            Case 28 'Hex => 0x1C
                status_response = "Incomplete Logo Stored in FLASH"
            Case Else 'this data is probably not status data, but a text response
                status_response = ""
        End Select
        
        'If the byte is a printable ASCII character display the value otherwise
        'display the status response
        If text(i) >= ASCII_SPACE And text(i) <= ASCII_TILDE Then
            message = message & Chr$(text(i))           'Build up text response string
        Else
            If message <> "" Then
                Display_Text (Len(message) & " Bytes returned as string = " & message)
                message = ""
            End If
            Display_Text ("Byte #" & counter & " is 0x" & Hex(text(i)) & " which is a " & status_response)
        End If
        counter = counter + 1
    Next i
End Sub
' This function will receive a string, check to see if the message text
' box is currently being used and append the string (and CRLF) to whatever is
' currently displayed in the message box
Public Sub Display_Text(str As String)

    'append new string to old text and throw in CR LF
    If FrmBoca.txtmsg.Visible = True Then
        FrmBoca.txtmsg.text = FrmBoca.txtmsg.text & str & Chr$(13) & Chr$(10)
    End If
    
End Sub
'Clear the text from the message box
Public Sub Clear_Display()
    FrmBoca.txtmsg.text = ""        'Clear out text
End Sub
'**************************************************************************************
'*
'* Routine Name  : pick_usb()
'*
'* Establish location of a Boca Systems USB printer, should one be connected.
'* if auto=True then index is irrelavent, a printer will be automatically selected
'* if auto=False, then the printer selected is based on the index.
'* if three printers found the indexes will run from 0 to 2.  The device numbers
'* displayed on the screen will run from 1 to 3, just to make it easier on the
'* user.  The index value is (device number - 1).
'**************************************************************************************
Public Sub Pick_USB(ByVal auto As Boolean, ByVal index As Integer)

    Dim temp(0 To 1) As Byte
    Dim ptr As Long              'Pointer to read/write text buffer
    Dim ptr1, ptr2 As String

    
    'if the number entered is out of range, simply pick the first printer found
    If ((index < 0) Or (index >= USB_printer_count)) Then
        index = 0
    End If
    
    Call Shutdown       'Close any open USB channel
    ptr1 = auto
    ptr2 = index
    ptr_USB = Open_USB(ByVal ptr1, ByVal ptr2)   'Call DLL function to open a channel to the printer
    
    'If a USB Boca printer was opened then establish R/W timeout periods
    'for the USB interface.
    If ptr_USB <> 0 Then

        current_printer = index + 1
        Printer_Table(current_printer).hUSB = ptr_USB
        
        'set write time out
        temp(0) = 119            'lower case w is ascii 119
        ptr = VarPtr(temp(0))
        ptr2 = 3000
        Call Timeout_USB(ByVal ptr, ByVal ptr2) '"3000")         ' 3 second write timeout

        'set read time out
        temp(0) = 114            'lower case r is ascii 114
        ptr = VarPtr(temp(0))
        ptr2 = 500
        Call Timeout_USB(ByVal ptr, ByVal ptr2) '"500")          ' 1/2 second read timeout
        
    End If

End Sub


Public Sub Scan_USB()
    
    Dim Name As String * 64        'Product Name
    Dim i As Integer
    
    CR = Chr$(13)       'Use this for a carriage return
    LF = Chr$(10)       'Use this for a line feed
    TB = Chr$(9)        'Use this for a horizontal Tab
    
    Call Shutdown       'Close any open USB channel
    
    pTable = VarPtr(Printer_Table(1))       'set pointer to start of table
    USB_printer_count = 0                   'initialize the counter to zero
    USB_printer_count = Find_Printers(ByVal pTable)  'Populate table, Count & identify USB printers
    
    'If a user has more than one printer enable the button to allow choices.
    If USB_printer_count > 1 Then
        FrmBoca.CmdPick.Enabled = True
    End If
    
    FrmBoca.Show        'Display the form and show what printers were found
    
    Display_Text ("Found " & USB_printer_count & " Boca Systems USB printer(s)")
    If USB_printer_count > 0 Then
    
        Display_Text (CR & LF & "Device" & TB & "Serial")
        Display_Text ("Number" & TB & "Number" & TB & "Name")
        Display_Text ("======" & TB & "======" & TB & "==================")
        For i = 1 To USB_printer_count
        
            Select Case Printer_Table(i).Product_ID
    
                Case 1      '0x0001
                    Name = "Model 42 - FGL/PCL4"
                Case 257    '0x0101
                    Name = "Model 44 - FGL"
                Case 258    '0x0102
                    Name = "Model 44 - PCL4"
                Case Else   'Future models
                    Name = " "
        
            End Select
            Display_Text (i & TB & Format$(Printer_Table(i).Serial_Number, "000000") & TB & Name)             'pointer to printer config

        Next i
        
    End If
    
End Sub



