// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the BOCA1_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// BOCA1_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef BOCA_DLL_EXPORTS
#define BOCA1_API __declspec(dllexport)
#else
#define BOCA1_API __declspec(dllimport)
#endif

/* End of global definitions */
#pragma comment(lib, "boca.lib")

#define WD_USB_MAX_PIPE_NUMBER 32
#define WD_USB_MAX_ENDPOINTS WD_USB_MAX_PIPE_NUMBER
#define WD_USB_MAX_INTERFACES 30
#define WD_USB_MAX_DEVICE_NUMBER 127
#define LPT1_S "LPT1:"
#define LPT2_S "LPT2:"
#define LPT3_S "LPT3:"
#define LPT4_S "LPT4:"
#define COM1_S "COM1:"
#define COM2_S "COM2:"
#define COM3_S "COM3:"
#define USB_S  "USB"

enum { PACKET_SIZE  = 256 };
enum { USB_PRINTER_MAX  = 127 };
enum { USB_PACKET_SIZE  = 256 };
enum { SER_PACKET_SIZE  = 128 };
enum { PAR_PACKET_SIZE  = 128 };
enum { USB_DEFAULT_VENDOR_ID  = 0x0A43 };
enum { USB_DEFAULT_PRODUCT_ID = 0x0001 };
enum { USB_DEFAULT_CONFIG = 0 };
enum { USB_DEFAULT_INTERFACE = 0 };
enum { USB_DEFAULT_ALTERNATE = 0 };
enum { READ_PIPE = 130};
enum { WRITE_PIPE = 1};
enum { USB  = 1};
enum { BOCA_ID  = 0x0A43 };
enum { SPACE = 32 };
enum { TILDE = 126 };
enum { OPEN_DELIMETER = 60 };		//Open Bracket -> "<"
enum { SET_READ_TIMEOUT = 114 };	//Read timeout function number
enum { SET_WRITE_TIMEOUT = 119 };	//Write timeout function number

// Types to be used later.
	typedef unsigned long       DWORD;
	typedef int                 BOOL;
	typedef unsigned char       BYTE;
	typedef unsigned short      WORD;
	typedef float               FLOAT;
	typedef void				*PVOID;
	typedef PVOID				HANDLE;

	typedef struct
	{
		DWORD dwVendorId;
		DWORD dwProductId;
	} WD_USB_ID;

	typedef struct
	{
		DWORD dwNumber;			// Pipe 0 is the default pipe
		DWORD dwMaximumPacketSize;
		DWORD type;				// USB_PIPE_TYPE
		DWORD direction;		// USB_DIR
								// Isochronous, Bulk, Interrupt are either USB_DIR_IN or USB_DIR_OUT
								// Control are USB_DIR_IN_OUT
		DWORD dwInterval;		// interval in ms relevant to Interrupt pipes
	} WD_USB_PIPE_INFO;

	typedef struct
	{
		DWORD dwPipes;
		WD_USB_PIPE_INFO Pipe[WD_USB_MAX_PIPE_NUMBER];
	} WD_USB_DEVICE_INFO;


	typedef struct USB_STRUCT
	{
		HANDLE              hWD;
		WD_USB_DEVICE_INFO  deviceInfo;
		DWORD               hDevice;
	} USB_STRUCT;

	typedef struct USB_STRUCT *USB_HANDLE;

	//DO NOT CHANGE THIS STRUCTURE
	//This structure is used by the DLL. 
	typedef struct
	{
		USB_HANDLE	hUSB;
		DWORD		Serial_Number;
		DWORD		Product_ID;
	} USB_PRINTER;

	typedef struct
	{
		char		Name[64];
	} PRINTER_NAME;
	

/**************************************************************************************
 * Prototype Definitions of Dynamic Link Library routines. Exporting all of these 
 * functions from boca.dll.
 **************************************************************************************/
extern "C" BOCA1_API DWORD Write_USB(USB_HANDLE hUSB, PVOID pBuffer, DWORD dwSize,int pipe);
extern "C" BOCA1_API DWORD Read_USB(USB_HANDLE hUSB, PVOID pBuffer, DWORD dwSize,int pipe);
extern "C" BOCA1_API void RegisterWinDriver();
extern "C" BOCA1_API BOOL USB_Get_WD_handle(HANDLE *phWD);
extern "C" BOCA1_API void USB_Printer_choice(DWORD dwVendorId, DWORD dwProductId);
extern "C" BOCA1_API USB_HANDLE USB_LocateAndOpenDevice(BOOL fUseDefault,int Index);
extern "C" BOCA1_API int proc_address(HINSTANCE hinstLib);
extern "C" BOCA1_API void USB_Timeout(PVOID pBuffer, int timeout);
extern "C" BOCA1_API int USB_Find(USB_PRINTER *Printer_Table);
//extern "C" BOCA1_API int USB_Find(int *Printer_Table);
extern "C" BOCA1_API void USB_Close(USB_HANDLE hUSB);
extern "C" BOCA1_API BOOL USB_Get_WD_handle(HANDLE *phWD);
extern "C" BOCA1_API DWORD Status_USB(USB_HANDLE hUSB);
extern "C" BOCA1_API DWORD Control_USB(USB_HANDLE hUSB, PVOID pBuffer, DWORD dwSize, CHAR setupPacket[8]);

/* Type definitions as pointers to functions with parameters */
typedef DWORD (*Read_USB_type)(USB_HANDLE hUSB, PVOID pBuffer, DWORD dwSize,int pipe);
typedef DWORD (*Write_USB_type)(USB_HANDLE hUSB, PVOID pBuffer, DWORD dwSize,int pipe);
typedef void  (*RegisterWinDriver_type)();
typedef void (*USB_Printer_choice_type)(DWORD dwVendorId, DWORD dwProductId);
typedef USB_HANDLE (*USB_LocateAndOpenDevice_type)(BOOL fUseDefault,int Index);
typedef int (*proc_address_type)(HINSTANCE hinstLib);
typedef void (*USB_Timeout_type)(PVOID pBuffer, int timeout);
typedef int (*USB_Find_type)(USB_PRINTER *Printer_Table);
//typedef int (*USB_Find_type)(int *Printer_Table);
typedef void (*USB_Close_type)(USB_HANDLE hUSB);
typedef BOOL (*USB_Get_WD_handle_type)(HANDLE *phWD);
typedef DWORD (*Status_USB_type)(USB_HANDLE hUSB);
typedef DWORD (*Control_USB_type)(USB_HANDLE hUSB, PVOID pBuffer, DWORD dwSize, CHAR setupPacket[8]);


/*  Global Definitions for all DLL routines */

extern "C" BOCA1_API  Read_USB_type Read_Data;
extern "C" BOCA1_API  Write_USB_type Write_Data;
extern "C" BOCA1_API  RegisterWinDriver_type Register_Driver;
extern "C" BOCA1_API  USB_Printer_choice_type Choose_Printer;
extern "C" BOCA1_API  USB_LocateAndOpenDevice_type Locate_Device;
extern "C" BOCA1_API  USB_Timeout_type Timeout_Port;
extern "C" BOCA1_API  USB_Find_type Find_Printers;
extern "C" BOCA1_API  USB_Close_type Close_Port;
extern "C" BOCA1_API  USB_Get_WD_handle_type Get_Handle;
extern "C" BOCA1_API  Status_USB_type USB_Status;
extern "C" BOCA1_API  Control_USB_type USB_Control;
